@@ -1,6 +1,11 @@
 # flake8: noqa
 
-from pandas._libs import NaT, Period, Timedelta, Timestamp
+from pandas._libs import (
+    NaT,
+    Period,
+    Timedelta,
+    Timestamp,
+)
 from pandas._libs.missing import NA
 
 from pandas.core.dtypes.dtypes import (
@@ -9,12 +14,24 @@ from pandas.core.dtypes.dtypes import (
     IntervalDtype,
     PeriodDtype,
 )
-from pandas.core.dtypes.missing import isna, isnull, notna, notnull
+from pandas.core.dtypes.missing import (
+    isna,
+    isnull,
+    notna,
+    notnull,
+)
 
-from pandas.core.algorithms import factorize, unique, value_counts
+from pandas.core.algorithms import (
+    factorize,
+    unique,
+    value_counts,
+)
 from pandas.core.arrays import Categorical
 from pandas.core.arrays.boolean import BooleanDtype
-from pandas.core.arrays.floating import Float32Dtype, Float64Dtype
+from pandas.core.arrays.floating import (
+    Float32Dtype,
+    Float64Dtype,
+)
 from pandas.core.arrays.integer import (
     Int8Dtype,
     Int16Dtype,
@@ -28,7 +45,10 @@ from pandas.core.arrays.integer import (
 from pandas.core.arrays.string_ import StringDtype
 from pandas.core.construction import array
 from pandas.core.flags import Flags
-from pandas.core.groupby import Grouper, NamedAgg
+from pandas.core.groupby import (
+    Grouper,
+    NamedAgg,
+)
 from pandas.core.indexes.api import (
     CategoricalIndex,
     DatetimeIndex,
@@ -42,8 +62,14 @@ from pandas.core.indexes.api import (
     TimedeltaIndex,
     UInt64Index,
 )
-from pandas.core.indexes.datetimes import bdate_range, date_range
-from pandas.core.indexes.interval import Interval, interval_range
+from pandas.core.indexes.datetimes import (
+    bdate_range,
+    date_range,
+)
+from pandas.core.indexes.interval import (
+    Interval,
+    interval_range,
+)
 from pandas.core.indexes.period import period_range
 from pandas.core.indexes.timedeltas import timedelta_range
 from pandas.core.indexing import IndexSlice
